@@ -8,7 +8,7 @@
 //! Both types rely on run-time checks to enforce Rust's safety guarantees. `SendRc`
 //! requires the pointers to be disabled before being sent to a different thread, and
 //! re-enabled after. `SendOption` requires the option to be set to `None` before being
-//! sent to another thread, and then to be explicitly marked as sent.
+//! sent to another thread, and then to be explicitly migrated before use.
 //!
 //! `SendRc` is designed for constructing single-threaded hierarchies that support data
 //! sharing and interior mutability (`Cell` and `RefCell`), and which needs to be
@@ -24,12 +24,12 @@
 
 #![warn(missing_docs)]
 
-mod send_opt;
-mod send_rc;
-mod thread_id;
+pub mod send_opt;
+pub mod send_rc;
 
 #[cfg(feature = "deepsize")]
 mod deepsize;
+mod thread_id;
 
 pub use send_opt::SendOption;
-pub use send_rc::{PostSendRc, PreSendRc, SendRc};
+pub use send_rc::SendRc;
